@@ -4,8 +4,9 @@ A salt module for SSL/TLS.
 Can create a Certificate Authority (CA)
 or use Self-Signed certificates.
 
-:depends:   - PyOpenSSL Python module (0.10 or later, 0.14 or later for
-              X509 extension support)
+:depends: PyOpenSSL Python module (0.10 or later, 0.14 or later for X509
+    extension support)
+
 :configuration: Add the following values in /etc/salt/minion for the CA module
     to function properly:
 
@@ -14,7 +15,7 @@ or use Self-Signed certificates.
         ca.cert_base_path: '/etc/pki'
 
 
-CLI Example #1
+CLI Example #1:
 Creating a CA, a server request and its signed certificate:
 
 .. code-block:: bash
@@ -98,35 +99,39 @@ Create a server req + cert with non-CN filename for the cert
         cert_type=server cert_filename="something_completely_different"
     Created Certificate for "www.anothersometh.ing": /etc/pki/my_little/certs/something_completely_different.crt
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 # pylint: disable=C0103
 
-# Import python libs
+# Import Python libs
 import os
+import re
 import time
 import calendar
 import logging
 import math
 import binascii
-import salt.utils
-from salt._compat import string_types
-from salt.ext.six.moves import range as _range
 from datetime import datetime
-from distutils.version import LooseVersion  # pylint: disable=no-name-in-module
 
-import re
+# Import Salt libs
+import salt.utils.data
+import salt.utils.files
+import salt.utils.stringutils
+from salt.exceptions import CommandExecutionError
+from salt.utils.versions import LooseVersion as _LooseVersion
+
+# Import 3rd-party libs
+from salt.ext import six
+from salt.ext.six.moves import range as _range
 
 HAS_SSL = False
 X509_EXT_ENABLED = True
 try:
     import OpenSSL
     HAS_SSL = True
-    OpenSSL_version = LooseVersion(OpenSSL.__dict__.get('__version__', '0.0'))
+    OpenSSL_version = _LooseVersion(OpenSSL.__dict__.get('__version__', '0.0'))
 except ImportError:
     pass
 
-# Import salt libs
-
 
 log = logging.getLogger(__name__)
 
@@ -139,12 +144,12 @@ def __virtual__():
     Only load this module if the ca config options are set
     '''
     global X509_EXT_ENABLED
-    if HAS_SSL and OpenSSL_version >= LooseVersion('0.10'):
-        if OpenSSL_version < LooseVersion('0.14'):
+    if HAS_SSL and OpenSSL_version >= _LooseVersion('0.10'):
+        if OpenSSL_version < _LooseVersion('0.14'):
             X509_EXT_ENABLED = False
             log.debug('You should upgrade pyOpenSSL to at least 0.14.1 to '
                       'enable the use of X509 extensions in the tls module')
-        elif OpenSSL_version <= LooseVersion('0.15'):
+        elif OpenSSL_version <= _LooseVersion('0.15'):
             log.debug('You should upgrade pyOpenSSL to at least 0.15.1 to '
                       'enable the full use of X509 extensions in the tls module')
         # NOTE: Not having configured a cert path should not prevent this
@@ -225,25 +230,27 @@ def _new_serial(ca_name):
     '''
     hashnum = int(
         binascii.hexlify(
-            '{0}_{1}'.format(
-                _microtime(),
-                os.urandom(5))),
+            b'_'.join((
+                salt.utils.stringutils.to_bytes(_microtime()),
+                os.urandom(5) if six.PY3 else os.urandom(5).encode('hex')
+            ))
+        ),
         16
     )
-    log.debug('Hashnum: {0}'.format(hashnum))
+    log.debug('Hashnum: %s', hashnum)
 
     # record the hash somewhere
     cachedir = __opts__['cachedir']
-    log.debug('cachedir: {0}'.format(cachedir))
+    log.debug('cachedir: %s', cachedir)
     serial_file = '{0}/{1}.serial'.format(cachedir, ca_name)
     if not os.path.exists(cachedir):
         os.makedirs(cachedir)
     if not os.path.exists(serial_file):
-        fd = salt.utils.fopen(serial_file, 'w')
+        mode = 'w'
     else:
-        fd = salt.utils.fopen(serial_file, 'a+')
-    with fd as ofile:
-        ofile.write(str(hashnum))
+        mode = 'a+'
+    with salt.utils.files.fopen(serial_file, mode) as ofile:
+        ofile.write(str(hashnum))  # future lint: disable=blacklisted-function
 
     return hashnum
 
@@ -261,11 +268,8 @@ def _get_basic_info(ca_name, cert, ca_dir=None):
 
     index_file = "{0}/index.txt".format(ca_dir)
 
-    expire_date = _four_digit_year_to_two_digit(
-        datetime.strptime(
-            cert.get_notAfter(),
-            four_digit_year_fmt)
-    )
+    cert = _read_cert(cert)
+    expire_date = _four_digit_year_to_two_digit(_get_expiration_date(cert))
     serial_number = format(cert.get_serial_number(), 'X')
 
     # gotta prepend a /
@@ -306,8 +310,8 @@ def _write_cert_to_database(ca_name, cert, cacert_path=None, status='V'):
         subject
     )
 
-    with salt.utils.fopen(index_file, 'a+') as ofile:
-        ofile.write(index_data)
+    with salt.utils.files.fopen(index_file, 'a+') as ofile:
+        ofile.write(salt.utils.stringutils.to_str(index_data))
 
 
 def maybe_fix_ssl_version(ca_name, cacert_path=None, ca_filename=None):
@@ -343,20 +347,19 @@ def maybe_fix_ssl_version(ca_name, cacert_path=None, ca_filename=None):
         cert_base_path(),
         ca_name,
         ca_filename)
-    with salt.utils.fopen(certp) as fic:
+    with salt.utils.files.fopen(certp) as fic:
         cert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM,
                                                fic.read())
         if cert.get_version() == 3:
-            log.info(
-                'Regenerating wrong x509 version '
-                'for certificate {0}'.format(certp))
-            with salt.utils.fopen(ca_keyp) as fic2:
+            log.info('Regenerating wrong x509 version '
+                     'for certificate %s', certp)
+            with salt.utils.files.fopen(ca_keyp) as fic2:
                 try:
                     # try to determine the key bits
                     key = OpenSSL.crypto.load_privatekey(
                         OpenSSL.crypto.FILETYPE_PEM, fic2.read())
                     bits = key.bits()
-                except Exception:
+                except Exception:  # pylint: disable=broad-except
                     bits = 2048
                 try:
                     days = (datetime.strptime(
@@ -437,16 +440,15 @@ def get_ca(ca_name, as_text=False, cacert_path=None):
         salt '*' tls.get_ca test_ca as_text=False cacert_path=/etc/certs
     '''
     set_ca_path(cacert_path)
-    certp = '{0}/{1}/{2}_ca_cert.crt'.format(
+    certp = '{0}/{1}/{1}_ca_cert.crt'.format(
             cert_base_path(),
-            ca_name,
             ca_name)
     if not os.path.exists(certp):
         raise ValueError('Certificate does not exist for {0}'.format(ca_name))
     else:
         if as_text:
-            with salt.utils.fopen(certp) as fic:
-                certp = fic.read()
+            with salt.utils.files.fopen(certp) as fic:
+                certp = salt.utils.stringutils.to_unicode(fic.read())
     return certp
 
 
@@ -490,8 +492,8 @@ def get_ca_signed_cert(ca_name,
         raise ValueError('Certificate does not exists for {0}'.format(CN))
     else:
         if as_text:
-            with salt.utils.fopen(certp) as fic:
-                certp = fic.read()
+            with salt.utils.files.fopen(certp) as fic:
+                certp = salt.utils.stringutils.to_unicode(fic.read())
     return certp
 
 
@@ -539,30 +541,90 @@ def get_ca_signed_key(ca_name,
         raise ValueError('Certificate does not exists for {0}'.format(CN))
     else:
         if as_text:
-            with salt.utils.fopen(keyp) as fic:
-                keyp = fic.read()
+            with salt.utils.files.fopen(keyp) as fic:
+                keyp = salt.utils.stringutils.to_unicode(fic.read())
     return keyp
 
 
+def _read_cert(cert):
+    if isinstance(cert, six.string_types):
+        try:
+            with salt.utils.files.fopen(cert) as rfh:
+                return OpenSSL.crypto.load_certificate(
+                    OpenSSL.crypto.FILETYPE_PEM,
+                    rfh.read()
+                )
+        except Exception:  # pylint: disable=broad-except
+            log.exception('Failed to read cert from path %s', cert)
+            return None
+    else:
+        if not hasattr(cert, 'get_notAfter'):
+            log.error('%s is not a valid cert path/object', cert)
+            return None
+        else:
+            return cert
+
+
+def _get_expiration_date(cert):
+    '''
+    Returns a datetime.datetime object
+    '''
+    cert_obj = _read_cert(cert)
+
+    if cert_obj is None:
+        raise CommandExecutionError(
+            'Failed to read cert from {0}, see log for details'.format(cert)
+        )
+
+    return datetime.strptime(
+        salt.utils.stringutils.to_str(cert_obj.get_notAfter()),
+        four_digit_year_fmt
+    )
+
+
+def get_expiration_date(cert, date_format='%Y-%m-%d'):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Get a certificate's expiration date
+
+    cert
+        Full path to the certificate
+
+    date_format
+        By default this will return the expiration date in YYYY-MM-DD format,
+        use this to specify a different strftime format string. Note that the
+        expiration time will be in UTC.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' tls.get_expiration_date /path/to/foo.crt
+        salt '*' tls.get_expiration_date /path/to/foo.crt date_format='%d/%m/%Y'
+    '''
+    return _get_expiration_date(cert).strftime(date_format)
+
+
 def _check_onlyif_unless(onlyif, unless):
     ret = None
     retcode = __salt__['cmd.retcode']
     if onlyif is not None:
-        if not isinstance(onlyif, string_types):
+        if not isinstance(onlyif, six.string_types):
             if not onlyif:
-                ret = {'comment': 'onlyif execution failed', 'result': True}
-        elif isinstance(onlyif, string_types):
+                ret = {'comment': 'onlyif condition is false', 'result': True}
+        elif isinstance(onlyif, six.string_types):
             if retcode(onlyif) != 0:
-                ret = {'comment': 'onlyif execution failed', 'result': True}
-                log.debug('onlyif execution failed')
+                ret = {'comment': 'onlyif condition is false', 'result': True}
+                log.debug('onlyif condition is false')
     if unless is not None:
-        if not isinstance(unless, string_types):
+        if not isinstance(unless, six.string_types):
             if unless:
-                ret = {'comment': 'unless execution succeeded', 'result': True}
-        elif isinstance(unless, string_types):
+                ret = {'comment': 'unless condition is true', 'result': True}
+        elif isinstance(unless, six.string_types):
             if retcode(unless) == 0:
-                ret = {'comment': 'unless execution succeeded', 'result': True}
-                log.debug('unless execution succeeded')
+                ret = {'comment': 'unless condition is true', 'result': True}
+                log.debug('unless condition is true')
     return ret
 
 
@@ -575,7 +637,7 @@ def create_ca(ca_name,
               L='Salt Lake City',
               O='SaltStack',
               OU=None,
-              emailAddress='xyz@pdq.net',
+              emailAddress=None,
               fixmode=False,
               cacert_path=None,
               ca_filename=None,
@@ -605,7 +667,7 @@ def create_ca(ca_name,
     OU
         organizational unit, default is None
     emailAddress
-        email address for the CA owner, default is 'xyz@pdq.net'
+        email address for the CA owner, default is None
     cacert_path
         absolute path to ca certificates root directory
     ca_filename
@@ -671,17 +733,18 @@ def create_ca(ca_name,
     # try to reuse existing ssl key
     key = None
     if os.path.exists(ca_keyp):
-        with salt.utils.fopen(ca_keyp) as fic2:
+        with salt.utils.files.fopen(ca_keyp) as fic2:
             # try to determine the key bits
             try:
                 key = OpenSSL.crypto.load_privatekey(
                     OpenSSL.crypto.FILETYPE_PEM, fic2.read())
             except OpenSSL.crypto.Error as err:
                 log.warning('Error loading existing private key'
-                    ' %s, generating a new key: %s', ca_keyp, str(err))
+                            ' %s, generating a new key: %s',
+                            ca_keyp, err)
                 bck = "{0}.unloadable.{1}".format(ca_keyp,
                     datetime.utcnow().strftime("%Y%m%d%H%M%S"))
-                log.info('Saving unloadable CA ssl key in {0}'.format(bck))
+                log.info('Saving unloadable CA ssl key in %s', bck)
                 os.rename(ca_keyp, bck)
 
     if not key:
@@ -698,7 +761,8 @@ def create_ca(ca_name,
     if OU:
         ca.get_subject().OU = OU
     ca.get_subject().CN = CN
-    ca.get_subject().emailAddress = emailAddress
+    if emailAddress:
+        ca.get_subject().emailAddress = emailAddress
 
     ca.gmtime_adj_notBefore(0)
     ca.gmtime_adj_notAfter(int(days) * 24 * 60 * 60)
@@ -707,20 +771,20 @@ def create_ca(ca_name,
 
     if X509_EXT_ENABLED:
         ca.add_extensions([
-            OpenSSL.crypto.X509Extension('basicConstraints', True,
-                                         'CA:TRUE, pathlen:0'),
-            OpenSSL.crypto.X509Extension('keyUsage', True,
-                                         'keyCertSign, cRLSign'),
-            OpenSSL.crypto.X509Extension('subjectKeyIdentifier', False,
-                                         'hash', subject=ca)])
+            OpenSSL.crypto.X509Extension(
+                b'basicConstraints', True, b'CA:TRUE, pathlen:0'),
+            OpenSSL.crypto.X509Extension(
+                b'keyUsage', True, b'keyCertSign, cRLSign'),
+            OpenSSL.crypto.X509Extension(
+                b'subjectKeyIdentifier', False, b'hash', subject=ca)])
 
         ca.add_extensions([
             OpenSSL.crypto.X509Extension(
-                'authorityKeyIdentifier',
+                b'authorityKeyIdentifier',
                 False,
-                'issuer:always,keyid:always',
+                b'issuer:always,keyid:always',
                 issuer=ca)])
-    ca.sign(key, digest)
+    ca.sign(key, salt.utils.stringutils.to_str(digest))
 
     # always backup existing keys in case
     keycontent = OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM,
@@ -729,29 +793,35 @@ def create_ca(ca_name,
     if os.path.exists(ca_keyp):
         bck = "{0}.{1}".format(ca_keyp, datetime.utcnow().strftime(
             "%Y%m%d%H%M%S"))
-        with salt.utils.fopen(ca_keyp) as fic:
-            old_key = fic.read().strip()
+        with salt.utils.files.fopen(ca_keyp) as fic:
+            old_key = salt.utils.stringutils.to_unicode(fic.read()).strip()
             if old_key.strip() == keycontent.strip():
                 write_key = False
             else:
-                log.info('Saving old CA ssl key in {0}'.format(bck))
-                with salt.utils.fopen(bck, 'w') as bckf:
+                log.info('Saving old CA ssl key in %s', bck)
+                with salt.utils.files.fopen(bck, 'w') as bckf:
                     bckf.write(old_key)
                     os.chmod(bck, 0o600)
     if write_key:
-        with salt.utils.fopen(ca_keyp, 'w') as ca_key:
-            ca_key.write(keycontent)
+        with salt.utils.files.fopen(ca_keyp, 'wb') as ca_key:
+            ca_key.write(salt.utils.stringutils.to_bytes(keycontent))
 
-    with salt.utils.fopen(certp, 'w') as ca_crt:
+    with salt.utils.files.fopen(certp, 'wb') as ca_crt:
         ca_crt.write(
-            OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, ca))
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_certificate(
+                    OpenSSL.crypto.FILETYPE_PEM,
+                    ca
+                )
+            )
+        )
 
     _write_cert_to_database(ca_name, ca)
 
     ret = ('Created Private Key: "{0}/{1}/{2}.key." ').format(
         cert_base_path(), ca_name, ca_filename)
-    ret += ('Created CA "{0}": "{1}/{2}/{3}.crt."').format(
-        ca_name, cert_base_path(), ca_name, ca_filename)
+    ret += ('Created CA "{0}": "{1}/{0}/{2}.crt."').format(
+        ca_name, cert_base_path(), ca_filename)
 
     return ret
 
@@ -854,7 +924,7 @@ def create_csr(ca_name,
                L='Salt Lake City',
                O='SaltStack',
                OU=None,
-               emailAddress='xyz@pdq.net',
+               emailAddress=None,
                subjectAltName=None,
                cacert_path=None,
                ca_filename=None,
@@ -886,7 +956,7 @@ def create_csr(ca_name,
     OU
         organizational unit, default is None
     emailAddress
-        email address for the request, default is 'xyz@pdq.net'
+        email address for the request, default is None
     subjectAltName
         valid subjectAltNames in full form, e.g. to add DNS entry you would call
         this function with this value:
@@ -998,7 +1068,8 @@ def create_csr(ca_name,
     if OU:
         req.get_subject().OU = OU
     req.get_subject().CN = CN
-    req.get_subject().emailAddress = emailAddress
+    if emailAddress:
+        req.get_subject().emailAddress = emailAddress
 
     try:
         extensions = get_extensions(cert_type)['csr']
@@ -1006,21 +1077,31 @@ def create_csr(ca_name,
         extension_adds = []
 
         for ext, value in extensions.items():
-            extension_adds.append(OpenSSL.crypto.X509Extension(ext, False,
-                                                               value))
-
+            if isinstance(value, six.string_types):
+                value = salt.utils.stringutils.to_bytes(value)
+            extension_adds.append(
+                OpenSSL.crypto.X509Extension(
+                    salt.utils.stringutils.to_bytes(ext),
+                    False,
+                    value
+                )
+            )
     except AssertionError as err:
         log.error(err)
         extensions = []
 
     if subjectAltName:
         if X509_EXT_ENABLED:
-            if isinstance(subjectAltName, str):
+            if isinstance(subjectAltName, six.string_types):
                 subjectAltName = [subjectAltName]
 
             extension_adds.append(
                 OpenSSL.crypto.X509Extension(
-                    'subjectAltName', False, ", ".join(subjectAltName)))
+                    b'subjectAltName',
+                    False,
+                    b', '.join(salt.utils.data.encode(subjectAltName))
+                )
+            )
         else:
             raise ValueError('subjectAltName cannot be set as X509 '
                              'extensions are not supported in pyOpenSSL '
@@ -1031,20 +1112,26 @@ def create_csr(ca_name,
         req.add_extensions(extension_adds)
 
     req.set_pubkey(key)
-    req.sign(key, digest)
+    req.sign(key, salt.utils.stringutils.to_str(digest))
 
     # Write private key and request
-    with salt.utils.fopen('{0}/{1}.key'.format(csr_path,
-                                               csr_filename), 'w+') as priv_key:
+    with salt.utils.files.fopen('{0}/{1}.key'.format(csr_path,
+                                                     csr_filename),
+                                'wb+') as priv_key:
         priv_key.write(
-            OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM,
+                                               key)
+            )
         )
 
-    with salt.utils.fopen(csr_f, 'w+') as csr:
+    with salt.utils.files.fopen(csr_f, 'wb+') as csr:
         csr.write(
-            OpenSSL.crypto.dump_certificate_request(
-                OpenSSL.crypto.FILETYPE_PEM,
-                req
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_certificate_request(
+                    OpenSSL.crypto.FILETYPE_PEM,
+                    req
+                )
             )
         )
 
@@ -1070,7 +1157,7 @@ def create_self_signed_cert(tls_dir='tls',
                             L='Salt Lake City',
                             O='SaltStack',
                             OU=None,
-                            emailAddress='xyz@pdq.net',
+                            emailAddress=None,
                             cacert_path=None,
                             cert_filename=None,
                             digest='sha256',
@@ -1096,7 +1183,7 @@ def create_self_signed_cert(tls_dir='tls',
     OU
         organizational unit, default is None
     emailAddress
-        email address for the request, default is 'xyz@pdq.net'
+        email address for the request, default is None
     cacert_path
         absolute path to ca certificates root directory
     digest
@@ -1167,31 +1254,36 @@ def create_self_signed_cert(tls_dir='tls',
     if OU:
         cert.get_subject().OU = OU
     cert.get_subject().CN = CN
-    cert.get_subject().emailAddress = emailAddress
+    if emailAddress:
+        cert.get_subject().emailAddress = emailAddress
 
     cert.set_serial_number(_new_serial(tls_dir))
     cert.set_issuer(cert.get_subject())
     cert.set_pubkey(key)
-    cert.sign(key, digest)
+    cert.sign(key, salt.utils.stringutils.to_str(digest))
 
     # Write private key and cert
-    with salt.utils.fopen(
-        '{0}/{1}/certs/{2}.key'.format(cert_base_path(),
-                                       tls_dir, cert_filename),
-        'w+'
-    ) as priv_key:
+    priv_key_path = '{0}/{1}/certs/{2}.key'.format(cert_base_path(),
+                                                   tls_dir,
+                                                   cert_filename)
+    with salt.utils.files.fopen(priv_key_path, 'wb+') as priv_key:
         priv_key.write(
-            OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM,
+                                               key)
+            )
         )
 
-    with salt.utils.fopen('{0}/{1}/certs/{2}.crt'.format(cert_base_path(),
-                                                         tls_dir,
-                                                         cert_filename
-                                                         ), 'w+') as crt:
+    crt_path = '{0}/{1}/certs/{2}.crt'.format(cert_base_path(),
+                                              tls_dir,
+                                              cert_filename)
+    with salt.utils.files.fopen(crt_path, 'wb+') as crt:
         crt.write(
-            OpenSSL.crypto.dump_certificate(
-                OpenSSL.crypto.FILETYPE_PEM,
-                cert
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_certificate(
+                    OpenSSL.crypto.FILETYPE_PEM,
+                    cert
+                )
             )
         )
 
@@ -1352,13 +1444,13 @@ def create_ca_signed_cert(ca_name,
         maybe_fix_ssl_version(ca_name,
                               cacert_path=cacert_path,
                               ca_filename=ca_filename)
-        with salt.utils.fopen('{0}/{1}/{2}.crt'.format(cert_base_path(),
+        with salt.utils.files.fopen('{0}/{1}/{2}.crt'.format(cert_base_path(),
                                                        ca_name,
                                                        ca_filename)) as fhr:
             ca_cert = OpenSSL.crypto.load_certificate(
                 OpenSSL.crypto.FILETYPE_PEM, fhr.read()
             )
-        with salt.utils.fopen('{0}/{1}/{2}.key'.format(cert_base_path(),
+        with salt.utils.files.fopen('{0}/{1}/{2}.key'.format(cert_base_path(),
                                                        ca_name,
                                                        ca_filename)) as fhr:
             ca_key = OpenSSL.crypto.load_privatekey(
@@ -1372,7 +1464,7 @@ def create_ca_signed_cert(ca_name,
 
     try:
         csr_path = '{0}/{1}.csr'.format(cert_path, csr_filename)
-        with salt.utils.fopen(csr_path) as fhr:
+        with salt.utils.files.fopen(csr_path) as fhr:
             req = OpenSSL.crypto.load_certificate_request(
                 OpenSSL.crypto.FILETYPE_PEM,
                 fhr.read())
@@ -1406,7 +1498,7 @@ def create_ca_signed_cert(ca_name,
                     native_exts_obj,
                     i)
                 exts.append(ext)
-        except Exception:
+        except Exception:  # pylint: disable=broad-except
             log.error('X509 extensions are unsupported in pyOpenSSL '
                       'versions prior to 0.14. Upgrade required to '
                       'use extensions. Current version: {0}'.format(
@@ -1423,18 +1515,21 @@ def create_ca_signed_cert(ca_name,
 
     cert.add_extensions(exts)
 
-    cert.sign(ca_key, digest)
+    cert.sign(ca_key, salt.utils.stringutils.to_str(digest))
 
     cert_full_path = '{0}/{1}.crt'.format(cert_path, cert_filename)
 
-    with salt.utils.fopen(cert_full_path, 'w+') as crt:
+    with salt.utils.files.fopen(cert_full_path, 'wb+') as crt:
         crt.write(
-            OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, cert))
+            salt.utils.stringutils.to_bytes(
+                OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM,
+                                                cert)
+            )
+        )
 
     _write_cert_to_database(ca_name, cert)
 
-    return ('Created Certificate for "{0}": '
-            '"{1}/{2}.crt"').format(
+    return 'Created Certificate for "{0}": "{1}/{2}.crt"'.format(
         CN,
         cert_path,
         cert_filename
@@ -1485,9 +1580,8 @@ def create_pkcs12(ca_name, CN, passphrase='', cacert_path=None, replace=False):
         return 'Certificate "{0}" already exists'.format(CN)
 
     try:
-        with salt.utils.fopen('{0}/{1}/{2}_ca_cert.crt'.format(cert_base_path(),
-                                                               ca_name,
-                                                               ca_name)) as fhr:
+        with salt.utils.files.fopen('{0}/{1}/{1}_ca_cert.crt'.format(cert_base_path(),
+                                                                     ca_name)) as fhr:
             ca_cert = OpenSSL.crypto.load_certificate(
                 OpenSSL.crypto.FILETYPE_PEM,
                 fhr.read()
@@ -1496,14 +1590,14 @@ def create_pkcs12(ca_name, CN, passphrase='', cacert_path=None, replace=False):
         return 'There is no CA named "{0}"'.format(ca_name)
 
     try:
-        with salt.utils.fopen('{0}/{1}/certs/{2}.crt'.format(cert_base_path(),
+        with salt.utils.files.fopen('{0}/{1}/certs/{2}.crt'.format(cert_base_path(),
                                                              ca_name,
                                                              CN)) as fhr:
             cert = OpenSSL.crypto.load_certificate(
                 OpenSSL.crypto.FILETYPE_PEM,
                 fhr.read()
             )
-        with salt.utils.fopen('{0}/{1}/certs/{2}.key'.format(cert_base_path(),
+        with salt.utils.files.fopen('{0}/{1}/certs/{2}.key'.format(cert_base_path(),
                                                              ca_name,
                                                              CN)) as fhr:
             key = OpenSSL.crypto.load_privatekey(
@@ -1519,26 +1613,32 @@ def create_pkcs12(ca_name, CN, passphrase='', cacert_path=None, replace=False):
     pkcs12.set_ca_certificates([ca_cert])
     pkcs12.set_privatekey(key)
 
-    with salt.utils.fopen('{0}/{1}/certs/{2}.p12'.format(cert_base_path(),
+    with salt.utils.files.fopen('{0}/{1}/certs/{2}.p12'.format(cert_base_path(),
                                                          ca_name,
-                                                         CN), 'w') as ofile:
-        ofile.write(pkcs12.export(passphrase=passphrase))
+                                                         CN), 'wb') as ofile:
+        ofile.write(
+            pkcs12.export(
+                passphrase=salt.utils.stringutils.to_bytes(passphrase)
+            )
+        )
 
     return ('Created PKCS#12 Certificate for "{0}": '
-            '"{1}/{2}/certs/{3}.p12"').format(
+            '"{1}/{2}/certs/{0}.p12"').format(
         CN,
         cert_base_path(),
         ca_name,
-        CN
     )
 
 
-def cert_info(cert_path, digest='sha256'):
+def cert_info(cert, digest='sha256'):
     '''
     Return information for a particular certificate
 
-    cert_path
-        path to the cert file
+    cert
+        path to the certifiate PEM file or string
+
+        .. versionchanged:: 2018.3.4
+
     digest
         what digest to use for fingerprinting
 
@@ -1547,25 +1647,46 @@ def cert_info(cert_path, digest='sha256'):
     .. code-block:: bash
 
         salt '*' tls.cert_info /dir/for/certs/cert.pem
+
     '''
     # format that OpenSSL returns dates in
     date_fmt = '%Y%m%d%H%M%SZ'
+    if '-----BEGIN' not in cert:
+        with salt.utils.files.fopen(cert) as cert_file:
+            cert = cert_file.read()
+    cert = OpenSSL.crypto.load_certificate(
+        OpenSSL.crypto.FILETYPE_PEM,
+        cert
+    )
+
+    issuer = {}
+    for key, value in cert.get_issuer().get_components():
+        if isinstance(key, bytes):
+            key = salt.utils.stringutils.to_unicode(key)
+        if isinstance(value, bytes):
+            value = salt.utils.stringutils.to_unicode(value)
+        issuer[key] = value
+
+    subject = {}
+    for key, value in cert.get_subject().get_components():
+        if isinstance(key, bytes):
+            key = salt.utils.stringutils.to_unicode(key)
+        if isinstance(value, bytes):
+            value = salt.utils.stringutils.to_unicode(value)
+        subject[key] = value
 
-    with salt.utils.fopen(cert_path) as cert_file:
-        cert = OpenSSL.crypto.load_certificate(
-            OpenSSL.crypto.FILETYPE_PEM,
-            cert_file.read()
-        )
     ret = {
-        'fingerprint': cert.digest(digest),
-        'subject': dict(cert.get_subject().get_components()),
-        'issuer': dict(cert.get_issuer().get_components()),
+        'fingerprint': salt.utils.stringutils.to_unicode(
+            cert.digest(salt.utils.stringutils.to_str(digest))
+            ),
+        'subject': subject,
+        'issuer': issuer,
         'serial_number': cert.get_serial_number(),
         'not_before': calendar.timegm(time.strptime(
             str(cert.get_notBefore().decode(__salt_system_encoding__)),
             date_fmt)),
         'not_after': calendar.timegm(time.strptime(
-            cert.get_notAfter().decode('utf-8'),
+            cert.get_notAfter().decode(__salt_system_encoding__),
             date_fmt)),
     }
 
@@ -1573,21 +1694,35 @@ def cert_info(cert_path, digest='sha256'):
     if hasattr(cert, 'get_extension_count'):
         ret['extensions'] = {}
         for i in _range(cert.get_extension_count()):
-            ext = cert.get_extension(i)
-            ret['extensions'][ext.get_short_name()] = str(ext)
+            try:
+                ext = cert.get_extension(i)
+                key = salt.utils.stringutils.to_unicode(ext.get_short_name())
+                ret['extensions'][key] = str(ext).strip()
+            except AttributeError:
+                continue
 
     if 'subjectAltName' in ret.get('extensions', {}):
+        valid_entries = ('DNS', 'IP Address')
         valid_names = set()
-        for name in str(ret['extensions']['subjectAltName']).split(", "):
-            if not name.startswith('DNS:'):
+        for name in str(ret['extensions']['subjectAltName']).split(', '):
+            entry, name = name.split(':', 1)
+            if entry not in valid_entries:
                 log.error('Cert {0} has an entry ({1}) which does not start '
-                          'with DNS:'.format(cert_path, name))
+                          'with {2}'.format(ret['subject'], name, '/'.join(valid_entries)))
             else:
-                valid_names.add(name[4:])
-        ret['subject_alt_names'] = ' '.join(valid_names)
+                valid_names.add(name)
+        ret['subject_alt_names'] = list(valid_names)
 
     if hasattr(cert, 'get_signature_algorithm'):
-        ret['signature_algorithm'] = cert.get_signature_algorithm()
+        try:
+            value = cert.get_signature_algorithm()
+            if isinstance(value, bytes):
+                value = salt.utils.stringutils.to_unicode(value)
+            ret['signature_algorithm'] = value
+        except AttributeError:
+            # On py3 at least
+            # AttributeError: cdata 'X509 *' points to an opaque type: cannot read fields
+            pass
 
     return ret
 
@@ -1638,7 +1773,7 @@ def create_empty_crl(
         return 'CRL "{0}" already exists'.format(crl_file)
 
     try:
-        with salt.utils.fopen('{0}/{1}/{2}.crt'.format(
+        with salt.utils.files.fopen('{0}/{1}/{2}.crt'.format(
                 cert_base_path(),
                 ca_name,
                 ca_filename)) as fp_:
@@ -1646,7 +1781,7 @@ def create_empty_crl(
                 OpenSSL.crypto.FILETYPE_PEM,
                 fp_.read()
             )
-        with salt.utils.fopen('{0}/{1}/{2}.key'.format(
+        with salt.utils.files.fopen('{0}/{1}/{2}.key'.format(
                 cert_base_path(),
                 ca_name,
                 ca_filename)) as fp_:
@@ -1660,8 +1795,8 @@ def create_empty_crl(
     crl = OpenSSL.crypto.CRL()
     crl_text = crl.export(ca_cert, ca_key)
 
-    with salt.utils.fopen(crl_file, 'w') as f:
-        f.write(crl_text)
+    with salt.utils.files.fopen(crl_file, 'w') as f:
+        f.write(salt.utils.stringutils.to_str(crl_text))
 
     return 'Created an empty CRL: "{0}"'.format(crl_file)
 
@@ -1724,7 +1859,7 @@ def revoke_cert(
         cert_filename = '{0}'.format(CN)
 
     try:
-        with salt.utils.fopen('{0}/{1}/{2}.crt'.format(
+        with salt.utils.files.fopen('{0}/{1}/{2}.crt'.format(
                 cert_base_path(),
                 ca_name,
                 ca_filename)) as fp_:
@@ -1732,7 +1867,7 @@ def revoke_cert(
                 OpenSSL.crypto.FILETYPE_PEM,
                 fp_.read()
             )
-        with salt.utils.fopen('{0}/{1}/{2}.key'.format(
+        with salt.utils.files.fopen('{0}/{1}/{2}.key'.format(
                 cert_base_path(),
                 ca_name,
                 ca_filename)) as fp_:
@@ -1743,14 +1878,8 @@ def revoke_cert(
     except IOError:
         return 'There is no CA named "{0}"'.format(ca_name)
 
-    try:
-        client_cert = OpenSSL.crypto.load_certificate(
-            OpenSSL.crypto.FILETYPE_PEM,
-            salt.utils.fopen('{0}/{1}.crt'.format(
-                cert_path,
-                cert_filename)).read()
-        )
-    except IOError:
+    client_cert = _read_cert('{0}/{1}.crt'.format(cert_path, cert_filename))
+    if client_cert is None:
         return 'There is no client certificate named "{0}"'.format(CN)
 
     index_file, expire_date, serial_number, subject = _get_basic_info(
@@ -1775,8 +1904,9 @@ def revoke_cert(
         index_serial_subject)
 
     ret = {}
-    with salt.utils.fopen(index_file) as fp_:
+    with salt.utils.files.fopen(index_file) as fp_:
         for line in fp_:
+            line = salt.utils.stringutils.to_unicode(line)
             if index_r_data_pattern.match(line):
                 revoke_date = line.split('\t')[2]
                 try:
@@ -1804,8 +1934,9 @@ def revoke_cert(
 
     crl = OpenSSL.crypto.CRL()
 
-    with salt.utils.fopen(index_file) as fp_:
+    with salt.utils.files.fopen(index_file) as fp_:
         for line in fp_:
+            line = salt.utils.stringutils.to_unicode(line)
             if line.startswith('R'):
                 fields = line.split('\t')
                 revoked = OpenSSL.crypto.Revoked()
@@ -1830,8 +1961,8 @@ def revoke_cert(
             crl_file)
         return ret
 
-    with salt.utils.fopen(crl_file, 'w') as fp_:
-        fp_.write(crl_text)
+    with salt.utils.files.fopen(crl_file, 'w') as fp_:
+        fp_.write(salt.utils.stringutils.to_str(crl_text))
 
     return ('Revoked Certificate: "{0}/{1}.crt", '
             'serial number: {2}').format(
