@@ -1,45 +1,54 @@
 # -*- coding: utf-8 -*-
 
 # Import Python Libs
-import json
+from __future__ import absolute_import, print_function, unicode_literals
+import os
 import time
-from distutils.version import StrictVersion  # pylint: disable=import-error,no-name-in-module
+import threading
 
 # Import Salt Libs
+import salt.utils.json
+import salt.utils.stringutils
 from salt.netapi.rest_tornado import saltnado
-from unit.netapi.rest_tornado.test_handlers import SaltnadoTestCase
+from salt.utils.versions import StrictVersion
 
 # Import Salt Testing Libs
-from salttesting import skipIf
-from salttesting.helpers import ensure_in_syspath
+from tests.unit.netapi.test_rest_tornado import SaltnadoTestCase
+from tests.support.helpers import flaky
+from tests.support.unit import skipIf
 
-ensure_in_syspath('../../../')
-
-try:
-    import tornado
-    HAS_TORNADO = True
-except ImportError:
-    HAS_TORNADO = False
-
-try:
-    import zmq
-    from zmq.eventloop.ioloop import ZMQIOLoop
-    HAS_ZMQ_IOLOOP = True
-except ImportError:
-    HAS_ZMQ_IOLOOP = False
+# Import 3rd-party libs
+from salt.ext import six
+from salt.utils.zeromq import zmq, ZMQDefaultLoop as ZMQIOLoop
+HAS_ZMQ_IOLOOP = bool(zmq)
+
+
+class _SaltnadoIntegrationTestCase(SaltnadoTestCase):  # pylint: disable=abstract-method
+
+    @property
+    def opts(self):
+        return self.get_config('client_config', from_scratch=True)
+
+    @property
+    def mod_opts(self):
+        return self.get_config('minion', from_scratch=True)
 
 
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
 @skipIf(StrictVersion(zmq.__version__) < StrictVersion('14.0.1'), 'PyZMQ must be >= 14.0.1 to run these tests.')
-class TestSaltAPIHandler(SaltnadoTestCase):
-    def get_app(self):
-        application = tornado.web.Application([('/', saltnado.SaltAPIHandler)], debug=True)
+class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
 
-        application.auth = self.auth
-        application.opts = self.opts
+    def setUp(self):
+        super(TestSaltAPIHandler, self).setUp()
+        os.environ['ASYNC_TEST_TIMEOUT'] = '300'
+
+    def get_app(self):
+        urls = [('/', saltnado.SaltAPIHandler)]
+
+        application = self.build_tornado_app(urls)
 
         application.event_listener = saltnado.EventListener({}, self.opts)
+        self.application = application
         return application
 
     def test_root(self):
@@ -47,18 +56,13 @@ class TestSaltAPIHandler(SaltnadoTestCase):
         Test the root path which returns the list of clients we support
         '''
         response = self.fetch('/',
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                 )
         self.assertEqual(response.code, 200)
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['clients'],
-                         ['runner',
-                          'runner_async',
-                          'local_async',
-                          'local',
-                          'local_batch']
-                         )
+        response_obj = salt.utils.json.loads(response.body)
+        self.assertEqual(sorted(response_obj['clients']),
+                         ['local', 'local_async', 'runner', 'runner_async'])
         self.assertEqual(response_obj['return'], 'Welcome')
 
     def test_post_no_auth(self):
@@ -72,17 +76,18 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json']},
                               follow_redirects=False,
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                               )
         self.assertEqual(response.code, 302)
         self.assertEqual(response.headers['Location'], '/login')
 
     # Local client tests
-    @skipIf(True, 'to be reenabled when #23623 is merged')
+
+    @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_simple_local_post(self):
         '''
         Test a basic API of /
@@ -93,14 +98,20 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = salt.utils.json.loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        # If --proxy is set, it will cause an extra minion_id to be in the
+        # response. Since there's not a great way to know if the test
+        # runner's proxy minion is running, and we're not testing proxy
+        # minions here anyway, just remove it from the response.
+        response_obj['return'][0].pop('proxytest', None)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
     def test_simple_local_post_no_tgt(self):
         '''
@@ -112,76 +123,57 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], ["No minions matched the target. No command was sent, no jid was assigned."])
 
-    # local_batch tests
-    @skipIf(True, 'to be reenabled when #23623 is merged')
-    def test_simple_local_batch_post(self):
+    # local client request body test
+
+    @skipIf(True, 'Undetermined race condition in test. Temporarily disabled.')
+    def test_simple_local_post_only_dictionary_request(self):
         '''
-        Basic post against local_batch
+        Test a basic API of /
         '''
-        low = [{'client': 'local_batch',
+        low = {'client': 'local',
                 'tgt': '*',
                 'fun': 'test.ping',
-                }]
+              }
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+        response_obj = salt.utils.json.loads(response.body)
+        self.assertEqual(len(response_obj['return']), 1)
+        # If --proxy is set, it will cause an extra minion_id to be in the
+        # response. Since there's not a great way to know if the test
+        # runner's proxy minion is running, and we're not testing proxy
+        # minions here anyway, just remove it from the response.
+        response_obj['return'][0].pop('proxytest', None)
+        self.assertEqual(response_obj['return'][0], {'minion': True, 'sub_minion': True})
 
-    # local_batch tests
-    @skipIf(True, 'to be reenabled when #23623 is merged')
-    def test_full_local_batch_post(self):
+    def test_simple_local_post_invalid_request(self):
         '''
-        Test full parallelism of local_batch
+        Test a basic API of /
         '''
-        low = [{'client': 'local_batch',
-                'tgt': '*',
-                'fun': 'test.ping',
-                'batch': '100%',
-                }]
+        low = ["invalid request"]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=30,
                               )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
-
-    def test_simple_local_batch_post_no_tgt(self):
-        '''
-        Local_batch testing with no tgt
-        '''
-        low = [{'client': 'local_batch',
-                'tgt': 'minion_we_dont_have',
-                'fun': 'test.ping',
-                }]
-        response = self.fetch('/',
-                              method='POST',
-                              body=json.dumps(low),
-                              headers={'Content-Type': self.content_type_map['json'],
-                                       saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
-                              )
-        response_obj = json.loads(response.body)
-        self.assertEqual(response_obj['return'], [{}])
+        self.assertEqual(response.code, 400)
 
     # local_async tests
     def test_simple_local_async_post(self):
@@ -191,16 +183,22 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
+        try:
+            # If --proxy is set, it will cause an extra minion_id to be in the
+            # response. Since there's not a great way to know if the test
+            # runner's proxy minion is running, and we're not testing proxy
+            # minions here anyway, just remove it from the response.
+            ret[0]['minions'].remove('proxytest')
+        except ValueError:
+            pass
 
         # TODO: verify pub function? Maybe look at how we test the publisher
         self.assertEqual(len(ret), 1)
@@ -218,17 +216,24 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
+        try:
+            # If --proxy is set, it will cause an extra minion_id to be in the
+            # response. Since there's not a great way to know if the test
+            # runner's proxy minion is running, and we're not testing proxy
+            # minions here anyway, just remove it from the response.
+            ret[0]['minions'].remove('proxytest')
+            ret[1]['minions'].remove('proxytest')
+        except ValueError:
+            pass
 
         self.assertEqual(len(ret), 2)
         self.assertIn('jid', ret[0])
@@ -254,17 +259,24 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 ]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
         ret[1]['minions'] = sorted(ret[1]['minions'])
+        try:
+            # If --proxy is set, it will cause an extra minion_id to be in the
+            # response. Since there's not a great way to know if the test
+            # runner's proxy minion is running, and we're not testing proxy
+            # minions here anyway, just remove it from the response.
+            ret[0]['minions'].remove('proxytest')
+            ret[1]['minions'].remove('proxytest')
+        except ValueError:
+            pass
 
         self.assertEqual(len(ret), 3)  # make sure we got 3 responses
         self.assertIn('jid', ret[0])  # the first 2 are regular returns
@@ -280,13 +292,44 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], [{}])
 
+    @skipIf(True, 'Undetermined race condition in test. Temporarily disabled.')
+    def test_simple_local_post_only_dictionary_request_with_order_masters(self):
+        '''
+        Test a basic API of /
+        '''
+        low = {'client': 'local',
+                'tgt': '*',
+                'fun': 'test.ping',
+              }
+
+        self.application.opts['order_masters'] = True
+        self.application.opts['syndic_wait'] = 5
+
+        response = self.fetch('/',
+                              method='POST',
+                              body=salt.utils.json.dumps(low),
+                              headers={'Content-Type': self.content_type_map['json'],
+                                       saltnado.AUTH_TOKEN_HEADER: self.token['token']},
+                              connect_timeout=30,
+                              request_timeout=30,
+                              )
+        response_obj = salt.utils.json.loads(response.body)
+        self.application.opts['order_masters'] = []
+        self.application.opts['syndic_wait'] = 5
+        # If --proxy is set, it will cause an extra minion_id to be in the
+        # response. Since there's not a great way to know if the test runner's
+        # proxy minion is running, and we're not testing proxy minions here
+        # anyway, just remove it from the response.
+        response_obj[0]['return'].pop('proxytest', None)
+        self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
+
     # runner tests
     def test_simple_local_runner_post(self):
         low = [{'client': 'runner',
@@ -294,15 +337,23 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
+                              connect_timeout=30,
+                              request_timeout=300,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
-        self.assertEqual(set(response_obj['return'][0]), set(['minion', 'sub_minion']))
+        try:
+            # If --proxy is set, it will cause an extra minion_id to be in the
+            # response. Since there's not a great way to know if the test
+            # runner's proxy minion is running, and we're not testing proxy
+            # minions here anyway, just remove it from the response.
+            response_obj['return'][0].remove('proxytest')
+        except ValueError:
+            pass
+        self.assertEqual(sorted(response_obj['return'][0]), sorted(['minion', 'sub_minion']))
 
     # runner_async tests
     def test_simple_local_runner_async_post(self):
@@ -311,78 +362,70 @@ class TestSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               connect_timeout=10,
                               request_timeout=10,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertIn('return', response_obj)
         self.assertEqual(1, len(response_obj['return']))
         self.assertIn('jid', response_obj['return'][0])
         self.assertIn('tag', response_obj['return'][0])
 
 
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
+@flaky
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestMinionSaltAPIHandler(SaltnadoTestCase):
+class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
-        application = tornado.web.Application([(r"/minions/(.*)", saltnado.MinionSaltAPIHandler),
-                                               (r"/minions", saltnado.MinionSaltAPIHandler),
-                                               ], debug=True)
-
-        application.auth = self.auth
-        application.opts = self.opts
-
+        urls = [(r"/minions/(.*)", saltnado.MinionSaltAPIHandler),
+                (r"/minions", saltnado.MinionSaltAPIHandler),
+                ]
+        application = self.build_tornado_app(urls)
         application.event_listener = saltnado.EventListener({}, self.opts)
         return application
 
+    @skipIf(True, 'issue #34753')
     def test_get_no_mid(self):
         response = self.fetch('/minions',
                               method='GET',
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         # one per minion
         self.assertEqual(len(response_obj['return'][0]), 2)
         # check a single grain
-        for minion_id, grains in response_obj['return'][0].iteritems():
+        for minion_id, grains in six.iteritems(response_obj['return'][0]):
             self.assertEqual(minion_id, grains['id'])
 
-    @skipIf(True, 'to be reenabled when #23623 is merged')
+    @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_get(self):
         response = self.fetch('/minions/minion',
                               method='GET',
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               follow_redirects=False,
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(len(response_obj['return']), 1)
         self.assertEqual(len(response_obj['return'][0]), 1)
         # check a single grain
         self.assertEqual(response_obj['return'][0]['minion']['id'], 'minion')
 
     def test_post(self):
-        low = [{'tgt': '*',
+        low = [{'tgt': '*minion',
                 'fun': 'test.ping',
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -394,19 +437,17 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
     def test_post_with_client(self):
         # get a token for this test
         low = [{'client': 'local_async',
-                'tgt': '*',
+                'tgt': '*minion',
                 'fun': 'test.ping',
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         ret = response_obj['return']
         ret[0]['minions'] = sorted(ret[0]['minions'])
 
@@ -417,40 +458,34 @@ class TestMinionSaltAPIHandler(SaltnadoTestCase):
 
     def test_post_with_incorrect_client(self):
         '''
-        The /minions endpoint is async only, so if you try something else
+        The /minions endpoint is asynchronous only, so if you try something else
         make sure you get an error
         '''
         # get a token for this test
-        low = [{'client': 'local_batch',
+        low = [{'client': 'local',
                 'tgt': '*',
                 'fun': 'test.ping',
                 }]
         response = self.fetch('/minions',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
         self.assertEqual(response.code, 400)
 
 
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestJobsSaltAPIHandler(SaltnadoTestCase):
+class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
-        application = tornado.web.Application([(r"/jobs/(.*)", saltnado.JobsSaltAPIHandler),
-                                               (r"/jobs", saltnado.JobsSaltAPIHandler),
-                                               ], debug=True)
-
-        application.auth = self.auth
-        application.opts = self.opts
-
+        urls = [(r"/jobs/(.*)", saltnado.JobsSaltAPIHandler),
+                (r"/jobs", saltnado.JobsSaltAPIHandler),
+                ]
+        application = self.build_tornado_app(urls)
         application.event_listener = saltnado.EventListener({}, self.opts)
         return application
 
-    @skipIf(True, 'to be reenabled when #23623 is merged')
+    @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_get(self):
         # test with no JID
         self.http_client.fetch(self.get_url('/jobs'),
@@ -458,12 +493,11 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                                method='GET',
                                headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                                follow_redirects=False,
-                               request_timeout=10,  # wait up to 10s for this response-- jenkins seems to be slow
                                )
-        response = self.wait(timeout=10)
-        response_obj = json.loads(response.body)['return'][0]
+        response = self.wait(timeout=30)
+        response_obj = salt.utils.json.loads(response.body)['return'][0]
         try:
-            for jid, ret in response_obj.iteritems():
+            for jid, ret in six.iteritems(response_obj):
                 self.assertIn('Function', ret)
                 self.assertIn('Target', ret)
                 self.assertIn('Target-type', ret)
@@ -471,20 +505,19 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
                 self.assertIn('StartTime', ret)
                 self.assertIn('Arguments', ret)
         except AttributeError as attribute_error:
-            print json.loads(response.body)
+            print(salt.utils.json.loads(response.body))
             raise
 
         # test with a specific JID passed in
-        jid = response_obj.iterkeys().next()
+        jid = next(six.iterkeys(response_obj))
         self.http_client.fetch(self.get_url('/jobs/{0}'.format(jid)),
                                self.stop,
                                method='GET',
                                headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                                follow_redirects=False,
-                               request_timeout=10,  # wait up to 10s for this response-- jenkins seems to be slow
                                )
-        response = self.wait(timeout=10)
-        response_obj = json.loads(response.body)['return'][0]
+        response = self.wait(timeout=30)
+        response_obj = salt.utils.json.loads(response.body)['return'][0]
         self.assertIn('Function', response_obj)
         self.assertIn('Target', response_obj)
         self.assertIn('Target-type', response_obj)
@@ -496,20 +529,16 @@ class TestJobsSaltAPIHandler(SaltnadoTestCase):
 
 # TODO: run all the same tests from the root handler, but for now since they are
 # the same code, we'll just sanity check
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestRunSaltAPIHandler(SaltnadoTestCase):
+class TestRunSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
-        application = tornado.web.Application([("/run", saltnado.RunSaltAPIHandler),
-                                               ], debug=True)
-
-        application.auth = self.auth
-        application.opts = self.opts
-
+        urls = [("/run", saltnado.RunSaltAPIHandler),
+                ]
+        application = self.build_tornado_app(urls)
         application.event_listener = saltnado.EventListener({}, self.opts)
         return application
 
-    @skipIf(True, 'to be reenabled when #23623 is merged')
+    @skipIf(True, 'to be re-enabled when #23623 is merged')
     def test_get(self):
         low = [{'client': 'local',
                 'tgt': '*',
@@ -517,27 +546,22 @@ class TestRunSaltAPIHandler(SaltnadoTestCase):
                 }]
         response = self.fetch('/run',
                               method='POST',
-                              body=json.dumps(low),
+                              body=salt.utils.json.dumps(low),
                               headers={'Content-Type': self.content_type_map['json'],
                                        saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
-        response_obj = json.loads(response.body)
+        response_obj = salt.utils.json.loads(response.body)
         self.assertEqual(response_obj['return'], [{'minion': True, 'sub_minion': True}])
 
 
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestEventsSaltAPIHandler(SaltnadoTestCase):
+class TestEventsSaltAPIHandler(_SaltnadoIntegrationTestCase):
     def get_app(self):
-        application = tornado.web.Application([(r"/events", saltnado.EventsSaltAPIHandler),
-                                               ], debug=True)
-
-        application.auth = self.auth
-        application.opts = self.opts
-
+        urls = [(r"/events", saltnado.EventsSaltAPIHandler),
+                ]
+        application = self.build_tornado_app(urls)
         application.event_listener = saltnado.EventListener({}, self.opts)
+
         # store a reference, for magic later!
         self.application = application
         self.events_to_fire = 0
@@ -548,14 +572,14 @@ class TestEventsSaltAPIHandler(SaltnadoTestCase):
         response = self.fetch('/events',
                               headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
                               streaming_callback=self.on_event,
-                              connect_timeout=10,
-                              request_timeout=10,
                               )
 
     def _stop(self):
         self.stop()
 
     def on_event(self, event):
+        if six.PY3:
+            event = event.decode('utf-8')
         if self.events_to_fire > 0:
             self.application.event_listener.event.fire_event({
                 'foo': 'bar',
@@ -576,53 +600,57 @@ class TestEventsSaltAPIHandler(SaltnadoTestCase):
             self.assertTrue(data.startswith('data: '))
 
 
-@skipIf(HAS_TORNADO is False, 'Tornado must be installed to run these tests')
 @skipIf(HAS_ZMQ_IOLOOP is False, 'PyZMQ version must be >= 14.0.1 to run these tests.')
-class TestWebhookSaltAPIHandler(SaltnadoTestCase):
-    def get_app(self):
-        application = tornado.web.Application([(r"/hook(/.*)?", saltnado.WebhookSaltAPIHandler),
-                                               ], debug=True)
+class TestWebhookSaltAPIHandler(_SaltnadoIntegrationTestCase):
 
-        application.auth = self.auth
-        application.opts = self.opts
+    def get_app(self):
+
+        urls = [(r"/hook(/.*)?", saltnado.WebhookSaltAPIHandler),
+                ]
+
+        application = self.build_tornado_app(urls)
 
         self.application = application
 
         application.event_listener = saltnado.EventListener({}, self.opts)
         return application
 
+    @skipIf(True, 'Skipping until we can devote more resources to debugging this test.')
     def test_post(self):
-        def verify_event(future):
-            '''
-            Verify that the event fired on the master matches what we sent
-            '''
-            event = future.result()
+        self._future_resolved = threading.Event()
+        try:
+            def verify_event(future):
+                '''
+                Notify the threading event that the future is resolved
+                '''
+                self._future_resolved.set()
+
+            self._finished = False  # TODO: remove after some cleanup of the event listener
+
+            # get an event future
+            future = self.application.event_listener.get_event(self,
+                                                               tag='salt/netapi/hook',
+                                                               callback=verify_event)
+            # fire the event
+            response = self.fetch('/hook',
+                                  method='POST',
+                                  body='foo=bar',
+                                  headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
+                                  )
+            response_obj = salt.utils.json.loads(response.body)
+            self.assertTrue(response_obj['success'])
+            resolve_future_timeout = 60
+            self._future_resolved.wait(resolve_future_timeout)
+            try:
+                event = future.result()
+            except Exception as exc:  # pylint: disable=broad-except
+                self.fail('Failed to resolve future under {} secs: {}'.format(resolve_future_timeout, exc))
             self.assertEqual(event['tag'], 'salt/netapi/hook')
             self.assertIn('headers', event['data'])
-            self.assertEqual(event['data']['post'], {'foo': 'bar'})
-        # get an event future
-        self._finished = False  # TODO: remove after some cleanup of the event listener
-        event = self.application.event_listener.get_event(self,
-                                                          tag='salt/netapi/hook',
-                                                          callback=verify_event,
-                                                          )
-        # fire the event
-        response = self.fetch('/hook',
-                              method='POST',
-                              body='foo=bar',
-                              headers={saltnado.AUTH_TOKEN_HEADER: self.token['token']},
-                              connect_timeout=10,
-                              request_timeout=10,
-                              )
-        response_obj = json.loads(response.body)
-        self.assertTrue(response_obj['success'])
-
-
-if __name__ == '__main__':
-    from integration import run_tests  # pylint: disable=import-error
-    run_tests(TestEventsSaltAPIHandler,
-              TestJobsSaltAPIHandler,
-              TestMinionSaltAPIHandler,
-              TestRunSaltAPIHandler,
-              TestSaltAPIHandler,
-              TestWebhookSaltAPIHandler, needs_daemon=True)
+            self.assertEqual(
+                event['data']['post'],
+                {'foo': salt.utils.stringutils.to_bytes('bar')}
+            )
+        finally:
+            self._future_resolved.clear()
+            del self._future_resolved
