@@ -4,26 +4,29 @@ Integration tests for the docker_container states
 '''
 # Import Python Libs
 from __future__ import absolute_import, print_function, unicode_literals
+
 import errno
 import functools
 import logging
 import os
 import subprocess
+import sys
 import tempfile
 
 # Import Salt Testing Libs
-from tests.support.unit import skipIf
 from tests.support.case import ModuleCase
 from tests.support.docker import with_network, random_name
-from tests.support.paths import FILES, TMP
 from tests.support.helpers import destructiveTest, with_tempdir
 from tests.support.mixins import SaltReturnAssertsMixin
+from tests.support.runtests import RUNTIME_VARS
+from tests.support.unit import skipIf
 
 # Import Salt Libs
 import salt.utils.files
 import salt.utils.network
 import salt.utils.path
 from salt.exceptions import CommandExecutionError
+from salt.modules.config import DEFAULTS as _config_defaults
 
 # Import 3rd-party libs
 from salt.ext import six
@@ -63,11 +66,11 @@ class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
         '''
         '''
         # Create temp dir
-        cls.image_build_rootdir = tempfile.mkdtemp(dir=TMP)
+        cls.image_build_rootdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
         # Generate image name
         cls.image = random_name(prefix='salt_busybox_')
 
-        script_path = os.path.join(FILES, 'file/base/mkimage-busybox-static')
+        script_path = os.path.join(RUNTIME_VARS.BASE_FILES, 'mkimage-busybox-static')
         cmd = [script_path, cls.image_build_rootdir, cls.image]
         log.debug('Running \'%s\' to build busybox image', ' '.join(cmd))
         process = subprocess.Popen(
@@ -231,6 +234,33 @@ class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
         # it should not have changed
         self.assertTrue('state' not in ret['changes'])
 
+    @with_network(subnet='10.247.197.96/27', create=True)
+    @container_name
+    def test_running_no_changes_hostname_network(self, container_name, net):
+        '''
+        Test that changes are not detected when a hostname is specified for a container
+        on a custom network
+        '''
+        # Create a container
+        kwargs = {
+            'name': container_name,
+            'image': self.image,
+            'shutdown_timeout': 1,
+            'network_mode': net.name,
+            'networks': [net.name],
+            'hostname': 'foo'
+        }
+        ret = self.run_state('docker_container.running', **kwargs)
+        self.assertSaltTrueReturn(ret)
+
+        ret = self.run_state('docker_container.running', **kwargs)
+        self.assertSaltTrueReturn(ret)
+        # Discard the outer dict with the state compiler data to make below
+        # asserts easier to read/write
+        ret = ret[next(iter(ret))]
+        # Should be no changes
+        self.assertFalse(ret['changes'])
+
     @container_name
     def test_running_start_false_with_replace(self, name):
         '''
@@ -743,7 +773,7 @@ class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
         container_netinfo = self.run_function(
             'docker.inspect_container',
             [container_name]).get('NetworkSettings', {}).get('Networks', {})[nets[-1].name]
-        autoip_keys = self.minion_opts['docker.compare_container_networks']['automatic']
+        autoip_keys = _config_defaults['docker.compare_container_networks']['automatic']
         autoip_config = {
             x: y for x, y in six.iteritems(container_netinfo)
             if x in autoip_keys and y
@@ -969,7 +999,7 @@ class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
                 os.close(fd)
             except OSError as exc:
                 if exc.errno != errno.EBADF:
-                    raise exc
+                    six.reraise(*sys.exc_info())
             else:
                 self.addCleanup(os.remove, ret)
                 return ret
@@ -1121,6 +1151,12 @@ class DockerContainerTestCase(ModuleCase, SaltReturnAssertsMixin):
         Test to make sure that we fail a state when the container exits with
         nonzero status if failhard is set to True, and that we don't when it is
         set to False.
+
+        NOTE: We can't use RUNTIME_VARS.SHELL_FALSE_PATH here because the image
+        we build on-the-fly here is based on busybox and does not include
+        /usr/bin/false. Therefore, when the host machine running the tests
+        has /usr/bin/false, it will not exist in the container and the Docker
+        Engine API will cause an exception to be raised.
         '''
         ret = self.run_state(
             'docker_container.run',
